#include "CarAppl.h"

using Veins::TraCIMobilityAccess;
using Veins::AnnotationManagerAccess;

Define_Module(CarAppl);

void CarAppl::initialize(int stage) {
	BaseApplLayer::initialize(stage);

	if (stage == 0) {
		mobility = TraCIMobilityAccess().get(getParentModule());
		traci = mobility->getCommandInterface();
		traciVehicle = mobility->getVehicleCommandInterface();
		annotations = AnnotationManagerAccess().getIfExists();
		ASSERT(annotations);

		myId = getParentModule()->getIndex();

		haveForwarded = false;
		if (myId == 0) {
			scheduleAt(40, stopSelf);
		}

	}
}

void CarAppl::handleSelfMsg(cMessage* msg) {
	if (strcmp(msg->getFullName(), "stopSelf") == 0) {
		traciVehicle->setSpeed(0);

		Car2XMessage* wsm = new Car2XMessage();
		wsm->setChannelNumber(178);
		wsm->setOriginPosition(curPosition);

		sendDelayedDown(wsm, uniform(0, .1));

		EV << "sent jam warning message" << std::endl;

	}
}

void CarAppl::handleLowerMsg(cMessage* msg) {

	if (Car2XMessage* wsm = dynamic_cast<Car2XMessage*>(msg)) {

		// Visualize transmission
		if( wsm->getOriginPosition().x != 0 &&  wsm->getOriginPosition().y!=0) {
			annotations->scheduleErase(3.0, annotations->drawLine(mobility->getPositionAt(simTime()), wsm->getOriginPosition(), "red"));
		}

		traciVehicle->changeRoute("9", 999.9);

		wsm->setChannelNumber(178);
		wsm->setOriginPosition(mobility->getPositionAt(simTime()));

		sendDelayedDown(wsm, uniform(0, .1));

		EV << "got jam warning message" << std::endl;

	}
}

void CarAppl::handlePositionUpdate(cObject* obj) {
	curPosition = mobility->getPositionAt(simTime());
}

void CarAppl::finish() {
/*
 * You might want to have some statistics?
 *
 **/
}

CarAppl::~CarAppl() {

}
